@@ -27,8 +27,9 @@ from .source import GaussianPulse, ContinuousWave
 from .source import VolumeSource, PlaneWave, ModeSource, GaussianBeam
 
 # monitor
-from .monitor import AbstractFieldMonitor, FreqMonitor, TimeMonitor, FieldMonitor, FieldTimeMonitor
-from .monitor import Monitor, AbstractFluxMonitor, FluxMonitor, FluxTimeMonitor, ModeMonitor
+from .monitor import FreqMonitor, TimeMonitor, FieldMonitor, FieldTimeMonitor
+from .monitor import Monitor, FluxMonitor, FluxTimeMonitor, ModeMonitor
+from .monitor import ModeFieldMonitor
 
 # simulation
 from .simulation import Simulation
