@@ -211,6 +211,22 @@ class Tidy3dBaseModel(pydantic.BaseModel):
         """define < for getting unique indices based on hash."""
         return hash(self) < hash(other)
 
+    def __gt__(self, other):
+        """define > for getting unique indices based on hash."""
+        return hash(self) > hash(other)
+
+    def __le__(self, other):
+        """define <= for getting unique indices based on hash."""
+        return hash(self) <= hash(other)
+
+    def __ge__(self, other):
+        """define >= for getting unique indices based on hash."""
+        return hash(self) >= hash(other)
+
+    def __eq__(self, other):
+        """define == for checking whether two base models are equal unique indices based on hash."""
+        return hash(self) == hash(other)
+
     def _json_string(self, include_unset: bool = True) -> str:
         """Returns string representation of a :class:`Tidy3dBaseModel`.
 
@@ -225,7 +241,15 @@ class Tidy3dBaseModel(pydantic.BaseModel):
             Json-formatted string holding :class:`Tidy3dBaseModel` data.
         """
         exclude_unset = not include_unset
-        return self.json(indent=INDENT, exclude_unset=exclude_unset)
+
+        # put infinity and -infinity in quotes
+        tmp_string = "<<TEMPORARY_INFINITY_STRING>>"
+        json_string = self.json(indent=INDENT, exclude_unset=exclude_unset)
+        json_string = json_string.replace("-Infinity", tmp_string)
+        json_string = json_string.replace("Infinity", '"Infinity"')
+        json_string = json_string.replace(tmp_string, '"-Infinity"')
+
+        return json_string
 
 
 def add_type_field(cls):
