@@ -2,20 +2,19 @@
 """Classes for Storing Monitor and Simulation Data."""
 
 from abc import ABC, abstractmethod
-from typing import Dict, List, Union, Optional
+from typing import Dict, List, Union
 import logging
 
 import xarray as xr
 import numpy as np
 import h5py
 
-from .types import Numpy, Direction, Array, numpy_encoding, Literal, Ax, Coordinate, Symmetry
+from .types import Numpy, Direction, Array, numpy_encoding, Literal, Ax, Coordinate, Symmetry, Axis
 from .base import Tidy3dBaseModel
 from .simulation import Simulation
 from .grid import YeeGrid
-from .mode import ModeSpec
 from .viz import add_ax_if_none, equal_aspect
-from ..log import log, DataError
+from ..log import DataError
 
 # TODO: add warning if fields didnt fully decay
 
@@ -95,6 +94,8 @@ class Tidy3dData(Tidy3dBaseModel):
         string_value_bytes = hdf5_grp.get(string_key)
         if not string_value_bytes:
             return None
+        if isinstance(string_value_bytes, str):
+            return string_value_bytes
         return Tidy3dData.decode_bytes(string_value_bytes)
 
     @staticmethod
@@ -172,7 +173,7 @@ class MonitorData(Tidy3dData, ABC):
             Whether the other :class:`MonitorData` instance has the same data.
         """
         assert isinstance(other, MonitorData), "can only check eqality on two monitor data objects"
-        return np.all(self.values == self.values)
+        return np.all(self.values == other.values)
 
     def add_to_group(self, hdf5_grp) -> None:
         """Add data contents to an hdf5 group."""
@@ -517,7 +518,7 @@ class ScalarFieldData(AbstractScalarFieldData, FreqData):
 
     def normalize(self, source_freq_amps: Array[complex]) -> None:
         """normalize the values by the amplitude of the source."""
-        self.values /= 1 * source_freq_amps  # pylint: disable=no-member
+        self.values /= source_freq_amps  # pylint: disable=no-member
 
 
 class ScalarFieldTimeData(AbstractScalarFieldData, TimeData):
@@ -685,15 +686,15 @@ class ModeAmpsData(AbstractModeData):
     data_attrs: Dict[str, str] = {"units": "sqrt(W)", "long_name": "mode amplitudes"}
     type: Literal["ModeAmpsData"] = "ModeAmpsData"
 
-    _dims = ("direction", "mode_index", "f")
+    _dims = ("direction", "f", "mode_index")
 
     def normalize(self, source_freq_amps: Array[complex]) -> None:
         """normalize the values by the amplitude of the source."""
-        self.values /= 1 * source_freq_amps  # pylint: disable=no-member
+        self.values /= source_freq_amps[None, :, None]  # pylint: disable=no-member
 
 
 class ModeIndexData(AbstractModeData):
-    """Stores modal amplitdudes from a :class:`ModeMonitor`.
+    """Stores effective propagation index from a :class:`ModeMonitor`.
 
     Parameters
     ----------
@@ -702,8 +703,7 @@ class ModeIndexData(AbstractModeData):
     f : numpy.ndarray
         Frequency coordinates (Hz).
     values : numpy.ndarray
-        Complex-valued array of mode amplitude values
-        with shape ``values.shape=(len(direction), len(mode_index), len(f))``.
+        Complex-valued array of effective index.
 
     Example
     -------
@@ -713,15 +713,34 @@ class ModeIndexData(AbstractModeData):
     """
 
     values: Array[complex]
-    data_attrs: Dict[str, str] = {"units": "None", "long_name": "complex effective index"}
+    data_attrs: Dict[str, str] = {"units": "", "long_name": "effective index"}
     type: Literal["ModeIndexData"] = "ModeIndexData"
 
-    _dims = ("mode_index", "f")
+    _dims = ("f", "mode_index")
 
     def normalize(self, source_freq_amps: Array[complex]) -> None:
         """normalize the values by the amplitude of the source."""
         return
 
+    @property
+    def n_complex(self):
+        """Complex effective index."""
+        return self.data
+
+    @property
+    def n_eff(self):
+        """Get real part of effective index."""
+        _n_eff = self.data.real
+        _n_eff.attrs["long_name"] = "effective n"
+        return _n_eff
+
+    @property
+    def k_eff(self):
+        """Get imaginary part of effective index."""
+        _k_eff = self.data.imag
+        _k_eff.attrs["long_name"] = "effective k"
+        return _k_eff
+
 
 class ModeData(CollectionData):
     """Stores a collection of mode decomposition amplitudes and mode effective indexes for all
@@ -729,18 +748,18 @@ class ModeData(CollectionData):
 
     Parameters
     ----------
-    data_dict : Dict[str, :class:`ScalarFieldData`]
-        Mapping of field name (eg. 'Ex') to its scalar field data.
+    data_dict : Dict[str, :class:`AbstractModeData`]
+        Mapping of "n_complex" to :class:`ModeIndexData`, and "amps" to :class:`ModeAmpsData`.
 
     Example
     -------
     >>> f = np.linspace(1e14, 2e14, 1001)
-    >>> x = np.linspace(-1, 1, 10)
-    >>> y = np.linspace(-2, 2, 20)
-    >>> z = np.linspace(0, 0, 1)
-    >>> values = (1+1j) * np.random.random((len(x), len(y), len(z), len(f)))
-    >>> field = ScalarFieldData(values=values, x=x, y=y, z=z, f=f)
-    >>> data = FieldData(data_dict={'Ex': field, 'Ey': field})
+    >>> mode_index = np.arange(2)
+    >>> n_complex = (1+1j) * np.random.random((len(f), len(mode_index)))
+    >>> index_data = ModeIndexData(values=n_complex, f=f, mode_index=mode_index)
+    >>> amps = (1+1j) * np.random.random((2, len(f), len(mode_index)))
+    >>> amps_data = ModeIndexData(values=amps, f=f, mode_index=mode_index)
+    >>> data = ModeData(data_dict={'n_complex': index_data, 'amps': amps_data})
     """
 
     data_dict: Dict[str, AbstractModeData]
@@ -767,7 +786,7 @@ class ModeData(CollectionData):
         """Get real part of effective index."""
         scalar_data = self.data_dict.get("n_complex")
         if scalar_data:
-            return scalar_data.data.real
+            return scalar_data.n_eff
         return None
 
     @property
@@ -775,10 +794,39 @@ class ModeData(CollectionData):
         """Get imaginary part of effective index."""
         scalar_data = self.data_dict.get("n_complex")
         if scalar_data:
-            return scalar_data.data.imag
+            return scalar_data.k_eff
         return None
 
 
+class ScalarModeFieldData(ScalarFieldData, AbstractModeData):
+    """Like ScalarFieldData but with extra dimension ``mode_index``."""
+
+    type: Literal["ScalarModeFieldData"] = "ScalarModeFieldData"
+    _dims = ("x", "y", "z", "f", "mode_index")
+
+
+class ModeFieldData(AbstractFieldData):
+    """Like FieldData but with extra dimension ``mode_index``."""
+
+    data_dict: Dict[str, ScalarModeFieldData]
+    type: Literal["ModeFieldData"] = "ModeFieldData"
+
+    def sel_mode_index(self, mode_index):
+        """Return a FieldData at the selected mode index."""
+        if mode_index not in self.Ex.mode_index:
+            raise DataError("Requested 'mode_index' not stored in ModeFieldData.")
+
+        data_dict = {}
+        for field_name, scalar_data in self.data_dict.items():
+            scalar_dict = scalar_data.dict()
+            scalar_dict.pop("mode_index")
+            scalar_dict.pop("type")
+            scalar_dict["values"] = scalar_data.data.sel(mode_index=mode_index).values
+            data_dict[field_name] = ScalarFieldData(**scalar_dict)
+
+        return FieldData(data_dict=data_dict)
+
+
 # maps MonitorData.type string to the actual type, for MonitorData.from_file()
 DATA_TYPE_MAP = {
     "ScalarFieldData": ScalarFieldData,
@@ -790,10 +838,119 @@ DATA_TYPE_MAP = {
     "ModeAmpsData": ModeAmpsData,
     "ModeIndexData": ModeIndexData,
     "ModeData": ModeData,
+    "ModeFieldData": ModeFieldData,
+    "ScalarModeFieldData": ScalarModeFieldData,
 }
 
 
-class SimulationData(Tidy3dBaseModel):
+class AbstractSimulationData(Tidy3dBaseModel, ABC):
+    """Abstract class to store a simulation and some data associated with it."""
+
+    simulation: Simulation
+
+    @equal_aspect
+    @add_ax_if_none
+    # pylint:disable=too-many-arguments, too-many-locals, too-many-branches, too-many-statements
+    def plot_field_array(
+        self,
+        field_data: xr.DataArray,
+        axis: Axis,
+        position: float,
+        val: Literal["real", "imag", "abs"] = "real",
+        freq: float = None,
+        eps_alpha: float = 0.2,
+        robust: bool = True,
+        ax: Ax = None,
+        **patch_kwargs,
+    ) -> Ax:
+        """Plot the field data for a monitor with simulation plot overlayed.
+
+        Parameters
+        ----------
+        field_data: xr.DataArray
+            DataArray with the field data to plot.
+        axis: Axis
+            Axis normal to the plotting plane.
+        position: float
+            Position along the axis.
+        val : Literal['real', 'imag', 'abs'] = 'real'
+            Which part of the field to plot.
+        freq: float = None
+            Frequency at which the permittivity is evaluated at (if dispersive).
+            By default, chooses permittivity as frequency goes to infinity.
+        eps_alpha : float = 0.2
+            Opacity of the structure permittivity.
+            Must be between 0 and 1 (inclusive).
+        robust : bool = True
+            If specified, uses the 2nd and 98th percentiles of the data to compute the color limits.
+            This helps in visualizing the field patterns especially in the presence of a source.
+        ax : matplotlib.axes._subplots.Axes = None
+            matplotlib axes to plot on, if not specified, one is created.
+        **patch_kwargs
+            Optional keyword arguments passed to ``add_artist(patch, **patch_kwargs)``.
+
+        Returns
+        -------
+        matplotlib.axes._subplots.Axes
+            The supplied or created matplotlib axes.
+        """
+
+        # select the cross section data
+        axis_label = "xyz"[axis]
+        interp_kwarg = {axis_label: position}
+
+        if len(field_data.coords[axis_label]) > 1:
+            try:
+                field_data = field_data.interp(**interp_kwarg)
+
+            except Exception as e:
+                raise DataError(f"Could not interpolate data at {axis_label}={position}.") from e
+
+        # select the field value
+        if val not in ("real", "imag", "abs"):
+            raise DataError(f"'val' must be one of ``{'real', 'imag', 'abs'}``, given {val}")
+
+        if val == "real":
+            field_data = field_data.real
+        elif val == "imag":
+            field_data = field_data.imag
+        elif val == "abs":
+            field_data = abs(field_data)
+
+        if val == "abs":
+            cmap = "magma"
+            eps_reverse = False
+        else:
+            cmap = "RdBu"
+            eps_reverse = True
+
+        # plot the field
+        xy_coord_labels = list("xyz")
+        xy_coord_labels.pop(axis)
+        x_coord_label, y_coord_label = xy_coord_labels  # pylint:disable=unbalanced-tuple-unpacking
+        field_data.plot(ax=ax, x=x_coord_label, y=y_coord_label, robust=robust, cmap=cmap)
+
+        # plot the simulation epsilon
+        ax = self.simulation.plot_structures_eps(
+            freq=freq,
+            cbar=False,
+            alpha=eps_alpha,
+            reverse=eps_reverse,
+            ax=ax,
+            **{axis_label: position},
+            **patch_kwargs,
+        )
+
+        # set the limits based on the xarray coordinates min and max
+        x_coord_values = field_data.coords[x_coord_label]
+        y_coord_values = field_data.coords[y_coord_label]
+        ax.set_xlim(min(x_coord_values), max(x_coord_values))
+        ax.set_ylim(min(y_coord_values), max(y_coord_values))
+
+        return ax
+
+
+class SimulationData(AbstractSimulationData):
     """Holds :class:`Monitor` data associated with :class:`Simulation`.
 
     Parameters
@@ -810,7 +967,6 @@ class SimulationData(Tidy3dBaseModel):
         A boolean flag denoting whether the data has been normalized by the spectrum of a source.
     """
 
-    simulation: Simulation
     monitor_data: Dict[str, Tidy3dData]
     log_string: str = None
     diverged: bool = False
@@ -829,8 +985,10 @@ class SimulationData(Tidy3dBaseModel):
         log_str = self.log
         lines = log_str.split("\n")
         decay_lines = [l for l in lines if "field decay" in l]
-        final_decay_line = decay_lines[-1]
-        final_decay = float(final_decay_line.split("field decay: ")[-1])
+        final_decay = 1.0
+        if len(decay_lines) > 0:
+            final_decay_line = decay_lines[-1]
+            final_decay = float(final_decay_line.split("field decay: ")[-1])
         return final_decay
 
     def __getitem__(self, monitor_name: str) -> Union[Tidy3dDataArray, xr.Dataset]:
@@ -849,9 +1007,8 @@ class SimulationData(Tidy3dBaseModel):
             a collection data instance is returned.
             Otherwise, if it is a MonitorData instance, the xarray representation is returned.
         """
+        self.ensure_monitor_exists(monitor_name)
         monitor_data = self.monitor_data.get(monitor_name)
-        if not monitor_data:
-            raise DataError(f"monitor '{monitor_name}' not found")
         if isinstance(monitor_data, MonitorData):
             return monitor_data.data
         return monitor_data
@@ -863,8 +1020,8 @@ class SimulationData(Tidy3dBaseModel):
 
     def ensure_field_monitor(self, data_obj: Tidy3dData) -> None:
         """Raise exception if monitor isn't a field monitor."""
-        if not isinstance(data_obj, (FieldData, FieldTimeData)):
-            raise DataError(f"data_obj '{data_obj}' " "not a FieldData or FieldTimeData instance.")
+        if not isinstance(data_obj, (FieldData, FieldTimeData, ModeFieldData)):
+            raise DataError(f"data_obj '{data_obj}' " "not an AbstractFieldData instance.")
 
     def at_centers(self, field_monitor_name: str) -> xr.Dataset:
         """return xarray.Dataset representation of field monitor data
@@ -883,7 +1040,6 @@ class SimulationData(Tidy3dBaseModel):
         """
 
         # get the data
-        self.ensure_monitor_exists(field_monitor_name)
         field_monitor_data = self.monitor_data.get(field_monitor_name)
         self.ensure_field_monitor(field_monitor_data)
 
@@ -896,8 +1052,6 @@ class SimulationData(Tidy3dBaseModel):
         field_dataset = field_monitor_data.colocate(x=centers.x, y=centers.y, z=centers.z)
         return field_dataset
 
-    @equal_aspect
-    @add_ax_if_none
     # pylint:disable=too-many-arguments, too-many-locals, too-many-branches, too-many-statements
     def plot_field(
         self,
@@ -909,6 +1063,7 @@ class SimulationData(Tidy3dBaseModel):
         val: Literal["real", "imag", "abs"] = "real",
         freq: float = None,
         time: float = None,
+        mode_index: int = None,
         eps_alpha: float = 0.2,
         robust: bool = True,
         ax: Ax = None,
@@ -938,6 +1093,8 @@ class SimulationData(Tidy3dBaseModel):
             By default, chooses permittivity as frequency goes to infinity.
         time: float = None
             if monitor is a :class:`FieldTimeMonitor`, specifies the time (sec) to plot the field.
+        mode_index: int = None
+            if monitor is a :class:`ModeFieldMonitor`, specifies which mode index to plot.
         eps_alpha : float = 0.2
             Opacity of the structure permittivity.
             Must be between 0 and 1 (inclusive).
@@ -956,9 +1113,12 @@ class SimulationData(Tidy3dBaseModel):
         """
 
         # get the monitor data
-        self.ensure_monitor_exists(field_monitor_name)
         monitor_data = self.monitor_data.get(field_monitor_name)
         self.ensure_field_monitor(monitor_data)
+        if isinstance(monitor_data, ModeFieldData):
+            if mode_index is None:
+                raise DataError("'mode_index' must be supplied to plot a ModeFieldMonitor.")
+            monitor_data = monitor_data.sel_mode_index(mode_index=mode_index)
 
         # get the field data component
         if field_name == "int":
@@ -967,6 +1127,7 @@ class SimulationData(Tidy3dBaseModel):
             for field in ("Ex", "Ey", "Ez"):
                 field_data = monitor_data[field]
                 xr_data += abs(field_data) ** 2
+            val = "abs"
         else:
             monitor_data.ensure_member_exists(field_name)
             xr_data = monitor_data.data_dict.get(field_name).data
@@ -978,59 +1139,37 @@ class SimulationData(Tidy3dBaseModel):
             field_data = xr_data.sel(f=freq, method="nearest")
         elif "t" in xr_data.coords:
             if time is None:
-                raise DataError("'time' must be supplied to plot a FieldMonitor.")
+                raise DataError("'time' must be supplied to plot a FieldTimeMonitor.")
             field_data = xr_data.sel(t=time, method="nearest")
         else:
             raise DataError("Field data has neither time nor frequency data, something went wrong.")
 
-        # select the cross section data
-        axis, pos = self.simulation.parse_xyz_kwargs(x=x, y=y, z=z)
-        axis_label = "xyz"[axis]
-        interp_kwarg = {axis_label: pos}
-
-        if len(field_data.coords[axis_label]) > 1:
+        if x is None and y is None and z is None:
+            """If a planar monitor, infer x/y/z based on the plane position and normal."""
+            monitor = self.simulation.get_monitor_by_name(field_monitor_name)
             try:
-                field_data = field_data.interp(**interp_kwarg)
-
+                axis = monitor.geometry.size.index(0.0)
+                position = monitor.geometry.center[axis]
             except Exception as e:
-                raise DataError(f"Could not interpolate data at {axis_label}={pos}.") from e
-
-        # select the field value
-        if val not in ("real", "imag", "abs"):
-            raise DataError(f"'val' must be one of ``{'real', 'imag', 'abs'}``, given {val}")
-
-        if field_name != "int":
-            if val == "real":
-                field_data = field_data.real
-            elif val == "imag":
-                field_data = field_data.imag
-            elif val == "abs":
-                field_data = abs(field_data)
-
-        if val == "abs" or field_name == "int":
-            cmap = "magma"
+                raise ValueError(
+                    "If none of 'x', 'y' or 'z' is specified, monitor must have a "
+                    "zero-sized dimension"
+                ) from e
         else:
-            cmap = "RdBu"
+            axis, position = self.simulation.parse_xyz_kwargs(x=x, y=y, z=z)
 
-        # plot the field
-        xy_coord_labels = list("xyz")
-        xy_coord_labels.pop(axis)
-        x_coord_label, y_coord_label = xy_coord_labels  # pylint:disable=unbalanced-tuple-unpacking
-        field_data.plot(ax=ax, x=x_coord_label, y=y_coord_label, robust=robust, cmap=cmap)
-
-        # plot the simulation epsilon
-        ax = self.simulation.plot_structures_eps(
-            freq=freq, cbar=False, x=x, y=y, z=z, alpha=eps_alpha, ax=ax, **patch_kwargs
+        return self.plot_field_array(
+            field_data=field_data,
+            axis=axis,
+            position=position,
+            val=val,
+            freq=freq,
+            eps_alpha=eps_alpha,
+            robust=robust,
+            ax=ax,
+            **patch_kwargs,
         )
 
-        # set the limits based on the xarray coordinates min and max
-        x_coord_values = field_data.coords[x_coord_label]
-        y_coord_values = field_data.coords[y_coord_label]
-        ax.set_xlim(min(x_coord_values), max(x_coord_values))
-        ax.set_ylim(min(y_coord_values), max(y_coord_values))
-
-        return ax
-
     def normalize(self, normalize_index: int = 0):
         """Return a copy of the :class:`.SimulationData` object with data normalized by source.
 
@@ -1056,9 +1195,8 @@ class SimulationData(Tidy3dBaseModel):
         try:
             source = self.simulation.sources[normalize_index]
             source_time = source.source_time
-        except Exception:  # pylint:disable=broad-except
-            logging.warning(f"Could not locate source at normalize_index={normalize_index}.")
-            return self
+        except IndexError as e:
+            raise DataError(f"Could not locate source at normalize_index={normalize_index}.") from e
 
         source_time = source.source_time
         sim_data_norm = self.copy(deep=True)
@@ -1069,6 +1207,10 @@ class SimulationData(Tidy3dBaseModel):
             """normalize a monitor data instance using the source time parameters."""
             freqs = monitor_data.f
             source_freq_amps = source_time.spectrum(times, freqs, dt)
+            # We remove the user-defined phase from the normalization. Otherwise, with a single
+            # source, we would get the exact same fields regardless of the source_time phase.
+            # Instead we would like the field phase to be determined by the source_time phase.
+            source_freq_amps *= np.exp(-1j * source_time.phase)
             monitor_data.normalize(source_freq_amps)
 
         for monitor_data in sim_data_norm.monitor_data.values():
