@@ -1,6 +1,7 @@
 # pylint: disable=too-many-lines
 """ Container holding all information about simulation and its components"""
 from typing import Dict, Tuple, List, Set
+from functools import lru_cache
 
 import pydantic
 import numpy as np
@@ -15,10 +16,10 @@ from .validators import validate_mode_objects_symmetry
 from .geometry import Box
 from .types import Symmetry, Ax, Shapely, FreqBound, GridSize
 from .grid import Coords1D, Grid, Coords
-from .medium import Medium, MediumType, AbstractMedium
+from .medium import Medium, MediumType, AbstractMedium, PECMedium
 from .structure import Structure
 from .source import SourceType, PlaneWave
-from .monitor import MonitorType
+from .monitor import MonitorType, Monitor, FreqMonitor
 from .pml import PMLTypes, PML, Absorber
 from .viz import StructMediumParams, StructEpsParams, PMLParams, SymParams
 from .viz import add_ax_if_none, equal_aspect
@@ -29,9 +30,9 @@ from ..constants import C_0, MICROMETER, SECOND, pec_val, inf
 from ..log import log, Tidy3dKeyError, SetupError
 
 # for docstring examples
-from .geometry import Sphere, Cylinder, PolySlab  # pylint:disable=unused-import
-from .source import VolumeSource, GaussianPulse  # pylint:disable=unused-import
-from .monitor import FieldMonitor, FluxMonitor, Monitor, FreqMonitor  # pylint:disable=unused-import
+# from .geometry import Sphere, Cylinder, PolySlab  # pylint:disable=unused-import
+# from .source import VolumeSource, GaussianPulse  # pylint:disable=unused-import
+# from .monitor import FieldMonitor, FluxMonitor, Monitor, FreqMonitor  # pylint:disable=unused-import
 
 
 # minimum number of grid points allowed per central wavelength in a medium
@@ -52,6 +53,9 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
     Example
     -------
+    >>> from tidy3d import Sphere, Cylinder, PolySlab
+    >>> from tidy3d import VolumeSource, GaussianPulse
+    >>> from tidy3d import FieldMonitor, FluxMonitor
     >>> sim = Simulation(
     ...     size=(2.0, 2.0, 2.0),
     ...     grid_size=(0.1, 0.1, 0.1),
@@ -100,21 +104,21 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         units=MICROMETER,
     )
 
+    run_time: pydantic.PositiveFloat = pydantic.Field(
+        ...,
+        title="Run Time",
+        description="Total electromagnetic evolution time in seconds. "
+        "Note: If simulation 'shutoff' is specified, "
+        "simulation will terminate early when shutoff condition met. ",
+        units=SECOND,
+    )
+
     medium: MediumType = pydantic.Field(
         Medium(),
         title="Background Medium",
         description="Background medium of simulation, defaults to vacuum if not specified.",
     )
 
-    run_time: pydantic.NonNegativeFloat = pydantic.Field(
-        0.0,
-        title="Run Time",
-        description="Total electromagnetic evolution time in seconds. "
-        "Note: If simulation 'shutoff' is specified, "
-        "simulation will terminate early when shutoff condition met.",
-        units=SECOND,
-    )
-
     symmetry: Tuple[Symmetry, Symmetry, Symmetry] = pydantic.Field(
         (0, 0, 0),
         title="Symmetries",
@@ -189,7 +193,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
     """ Validating setup """
 
-    @pydantic.validator("pml_layers", always=True)
+    @pydantic.validator("pml_layers", always=True, allow_reuse=True)
     def set_none_to_zero_layers(cls, val):
         """if any PML layer is None, set it to an empty :class:`PML`."""
         return tuple(PML(num_layers=0) if pml is None else pml for pml in val)
@@ -244,11 +248,11 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         sim_bound_min, sim_bound_max = sim_box.bounds
         sim_bounds = list(sim_bound_min) + list(sim_bound_max)
 
-        for structure in val:
+        for istruct, structure in enumerate(val):
             struct_bound_min, struct_bound_max = structure.geometry.bounds
             struct_bounds = list(struct_bound_min) + list(struct_bound_max)
 
-            for istruct, (sim_val, struct_val) in enumerate(zip(sim_bounds, struct_bounds)):
+            for sim_val, struct_val in zip(sim_bounds, struct_bounds):
 
                 if np.isclose(sim_val, struct_val):
                     log.warning(
@@ -326,12 +330,12 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         medium_bg = values.get("medium")
         mediums = [medium_bg] + [structure.medium for structure in structures]
 
-        for monitor in val:
+        for monitor_index, monitor in enumerate(val):
             if not isinstance(monitor, FreqMonitor):
                 continue
 
             freqs = np.array(monitor.freqs)
-            for medium in mediums:
+            for medium_index, medium in enumerate(mediums):
 
                 # skip mediums that have no freq range (all freqs valid)
                 if medium.frequency_range is None:
@@ -340,11 +344,16 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
                 # make sure medium frequency range includes all monitor frequencies
                 fmin_med, fmax_med = medium.frequency_range
                 if np.any(freqs < fmin_med) or np.any(freqs > fmax_med):
+                    if medium_index == 0:
+                        medium_str = "The simulation background medium"
+                    else:
+                        medium_str = f"The medium associated with structures[{medium_index-1}]"
+
                     log.warning(
-                        f"A medium in the simulation:\n\n({medium})\n\nhas a frequency "
-                        "range that does not fully cover the frequencies of a monitor:"
-                        f"\n\n({monitor})\n\nThis can cause innacuracies in the "
-                        "recorded results."
+                        medium_str + f"has a frequency range: ({fmin_med:2e}, {fmax_med:2e}) (Hz)"
+                        "that does not fully cover the frequencies contained in "
+                        f"monitors[{monitor_index}]. "
+                        "This can cause innacuracies in the recorded results."
                     )
         return val
 
@@ -364,15 +373,16 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         freq_min = min([freq_range[0] for freq_range in source_ranges], default=0.0)
         freq_max = max([freq_range[1] for freq_range in source_ranges], default=0.0)
 
-        for monitor in val:
+        for monitor_index, monitor in enumerate(val):
             if not isinstance(monitor, FreqMonitor):
                 continue
 
             freqs = np.array(monitor.freqs)
             if np.any(freqs < freq_min) or np.any(freqs > freq_max):
                 log.warning(
-                    f"A monitor in the simulation:\n\n({monitor})\n\nhas frequencies "
-                    "outside of the simulation frequency range as defined by the sources."
+                    f"monitors[{monitor_index}] contains frequencies "
+                    f"outside of the simulation frequency range ({freq_min:2e}, {freq_max:2e})"
+                    "(Hz) as defined by the sources."
                 )
         return val
 
@@ -389,26 +399,31 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         grid_size = values.get("grid_size")
         mediums = [medium_bg] + [structure.medium for structure in structures]
 
-        for source in val:
+        for source_index, source in enumerate(val):
             fmin_src, fmax_src = source.source_time.frequency_range()
             f_average = (fmin_src + fmax_src) / 2.0
 
-            for medium in mediums:
+            for medium_index, medium in enumerate(mediums):
+
+                # min wavelength in PEC is meaningless and we'll get divide by inf errors
+                if isinstance(medium, PECMedium):
+                    continue
 
                 eps_material = medium.eps_model(f_average)
                 n_material, _ = medium.eps_complex_to_nk(eps_material)
                 lambda_min = C_0 / f_average / n_material
 
-                for dl in grid_size:
+                for grid_index, dl in enumerate(grid_size):
                     if isinstance(dl, float):
                         if dl > lambda_min / MIN_GRIDS_PER_WVL:
                             log.warning(
-                                f"One of the grid sizes with a value of {dl:.4f} (um) "
-                                "was detected as being too large when compared to the "
-                                "central wavelength of a source within one of the "
-                                f"simulation mediums {lambda_min:.4f} (um).  "
-                                "To avoid inaccuracies, it is reccomended the grid size is "
-                                "reduced."
+                                f"The grid step in {'xyz'[grid_index]} has a value of {dl:.4f} (um)"
+                                ", which was detected as being large when compared to the "
+                                f"central wavelength of sources[{source_index}] "
+                                f"within the simulation medium "
+                                f"associated with structures[{medium_index + 1}], given by "
+                                f"{lambda_min:.4f} (um). "
+                                "To avoid inaccuracies, it is reccomended the grid size is reduced."
                             )
                     # TODO: warn about nonuniform grid
 
@@ -457,10 +472,11 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
     """ Pre submit validation (before web.upload()) """
 
-    def validate_contents(self) -> None:
-        """Validate the fully initialized simulation is within allowed limits."""
+    def validate_pre_upload(self) -> None:
+        """Validate the fully initialized simulation is ok for upload to our servers."""
         self._validate_size()
         self._validate_monitor_size()
+        # self._validate_run_time()
 
     def _validate_size(self) -> None:
         """Ensures the simulation is within size limits before simulation is uploaded."""
@@ -505,9 +521,19 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
                 f"a maximum of {MAX_MONITOR_DATA_SIZE_BYTES:.2e} are allowed."
             )
 
+    def _validate_run_time(self) -> None:
+        """Ensures that the simulation run time is > 0."""
+
+        if not self.run_time > 0:
+            raise SetupError(
+                "The `Simulation.run_time` parameter was left at its default value of 0.0. "
+                "For running a simulation on our servers it must be set to > 0.0."
+            )
+
     """ Accounting """
 
     @property
+    @lru_cache()
     def mediums(self) -> Set[MediumType]:
         """Returns set of distinct :class:`AbstractMedium` in simulation.
 
@@ -521,6 +547,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         return list(medium_dict.keys())
 
     @property
+    @lru_cache()
     def medium_map(self) -> Dict[MediumType, pydantic.NonNegativeInt]:
         """Returns dict mapping medium to index in material.
         ``medium_map[medium]`` returns unique global index of :class:`AbstractMedium` in simulation.
@@ -641,11 +668,16 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
             params_updater = StructMediumParams(medium=medium, medium_map=medium_map)
             kwargs_struct = params_updater.update_params(**kwargs)
             if medium == self.medium:
-                kwargs_struct["edgecolor"] = "white"
-                kwargs_struct["facecolor"] = "white"
+                continue
             patch = PolygonPatch(shape, **kwargs_struct)
             ax.add_artist(patch)
         ax = self._set_plot_bounds(ax=ax, x=x, y=y, z=z)
+
+        # clean up the axis display
+        axis, position = self.parse_xyz_kwargs(x=x, y=y, z=z)
+        ax = self.add_ax_labels_lims(axis=axis, ax=ax)
+        ax.set_title(f"cross section at {'xyz'[axis]}={position:.2f}")
+
         return ax
 
     @staticmethod
@@ -666,6 +698,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         z: float = None,
         freq: float = None,
         cbar: bool = True,
+        reverse: bool = True,
         ax: Ax = None,
         **kwargs,
     ) -> Ax:
@@ -684,6 +717,8 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         freq : float = None
             Frequency to evaluate the relative permittivity of all mediums.
             If not specified, evaluates at infinite frequency.
+        reverse : bool = True
+            If ``True``, the highest permittivity is plotted in black; if ``False``: white.
         ax : matplotlib.axes._subplots.Axes = None
             Matplotlib axes to plot on, if not specified, one is created.
         **kwargs
@@ -708,15 +743,23 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         medium_shapes = self._filter_structures_plane(self.structures, x=x, y=y, z=z)
         for (medium, shape) in medium_shapes:
             eps = medium.eps_model(freq).real
-            params_updater = StructEpsParams(eps=eps, eps_min=eps_min, eps_max=eps_max)
+            params_updater = StructEpsParams(
+                eps=eps, eps_min=eps_min, eps_max=eps_max, reverse=reverse
+            )
             kwargs_struct = params_updater.update_params(**kwargs)
             if medium == self.medium:
-                kwargs_struct["edgecolor"] = "white"
+                continue
             patch = PolygonPatch(shape, **kwargs_struct)
             ax.add_artist(patch)
         if cbar:
             self._add_cbar(eps_min=eps_min, eps_max=eps_max, ax=ax)
         ax = self._set_plot_bounds(ax=ax, x=x, y=y, z=z)
+
+        # clean up the axis display
+        axis, position = self.parse_xyz_kwargs(x=x, y=y, z=z)
+        ax = self.add_ax_labels_lims(axis=axis, ax=ax)
+        ax.set_title(f"cross section at {'xyz'[axis]}={position:.2f}")
+
         return ax
 
     @equal_aspect
@@ -861,6 +904,16 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
             pml_thicknesses.append((thick_l, thick_r))
         return pml_thicknesses
 
+    @property
+    def bounds_pml(self) -> Tuple[Tuple[float, float, float], Tuple[float, float, float]]:
+        """Simulation bounds including the PML regions."""
+        pml_thick = self.pml_thicknesses
+        bounds_in = self.bounds
+        bounds_min = tuple((bmin - pml[0] for bmin, pml in zip(bounds_in[0], pml_thick)))
+        bounds_max = tuple((bmax + pml[1] for bmax, pml in zip(bounds_in[1], pml_thick)))
+
+        return (bounds_min, bounds_max)
+
     @equal_aspect
     @add_ax_if_none
     def plot_pml(
@@ -959,11 +1012,11 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         """
 
         axis, _ = self.parse_xyz_kwargs(x=x, y=y, z=z)
-        _, ((xmin, ymin), (xmax, ymax)) = self._pop_bounds(axis=axis)
-        _, (pml_thick_x, pml_thick_y) = self.pop_axis(self.pml_thicknesses, axis=axis)
+        _, (xmin, ymin) = self.pop_axis(self.bounds_pml[0], axis=axis)
+        _, (xmax, ymax) = self.pop_axis(self.bounds_pml[1], axis=axis)
 
-        ax.set_xlim(xmin - pml_thick_x[0], xmax + pml_thick_x[1])
-        ax.set_ylim(ymin - pml_thick_y[0], ymax + pml_thick_y[1])
+        ax.set_xlim(xmin, xmax)
+        ax.set_ylim(ymin, ymax)
         return ax
 
     @staticmethod
@@ -1005,6 +1058,8 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         background_shapes = []
         for medium, shape in shapes:
 
+            shape = Box.evaluate_inf_shape(shape)
+
             # loop through background_shapes (note: all background are non-intersecting or merged)
             for index, (_medium, _shape) in enumerate(background_shapes):
 
@@ -1190,7 +1245,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
             Number of yee cells in the simulation.
         """
 
-        return np.prod(self.grid.num_cells)
+        return np.prod(self.grid.num_cells, dtype=np.int64)
 
     @staticmethod
     def _add_pml_to_bounds(num_layers: Tuple[int, int], bounds: Coords1D):
@@ -1235,7 +1290,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         n_max, _ = AbstractMedium.eps_complex_to_nk(eps_max)
         return wvl_min / n_max
 
-    def min_sym_box(self, box: Box) -> Box:
+    def min_sym_box(self, box: Box) -> Box:  # pylint:disable=too-many-locals
         """Compute the smallest Box restricted to the first quadrant in the presence of symmetries
         that fully covers the original Box when symmetries are applied.
 
@@ -1252,20 +1307,26 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         """
 
         bounds_min, bounds_max = box.bounds
+        sim_bs_min, sim_bs_max = self.bounds_pml
         bmin_new, bmax_new = [], []
 
-        for (center, sym, bmin, bmax) in zip(self.center, self.symmetry, bounds_min, bounds_max):
+        zipped = zip(self.center, self.symmetry, bounds_min, bounds_max, sim_bs_min, sim_bs_max)
+        for (center, sym, bmin, bmax, sim_bmin, sim_bmax) in zipped:
             if sym == 0 or center < bmin:
-                bmin_new.append(bmin)
-                bmax_new.append(bmax)
+                bmin_tmp, bmax_tmp = bmin, bmax
             else:
                 if bmax < center:
-                    bmin_new.append(2 * center - bmax)
-                    bmax_new.append(2 * center - bmin)
+                    bmin_tmp = 2 * center - bmax
+                    bmax_tmp = 2 * center - bmin
                 else:
                     # bmin <= center <= bmax
-                    bmin_new.append(center)
-                    bmax_new.append(max(bmax, 2 * center - bmin))
+                    bmin_tmp = center
+                    bmax_tmp = max(bmax, 2 * center - bmin)
+            # Extend well past the simulation domain if needed, but truncate if original box
+            # is too large, specifically to avoid issues with inf.
+            sim_size = sim_bmax - sim_bmin
+            bmin_new.append(max(bmin_tmp, sim_bmin - sim_size))
+            bmax_new.append(min(bmax_tmp, sim_bmax + sim_size))
 
         return Box.from_bounds(bmin_new, bmax_new)
 
