@@ -29,6 +29,9 @@ ARROW_LENGTH_FACTOR = 0.4
 # this times ARROW_LENGTH gives width
 ARROW_WIDTH_FACTOR = 0.4
 
+# arrow width cannot be larger than this factor times the max of axis height and width
+MAX_ARROW_WIDTH_FACTOR = 0.02
+
 
 """ Decorators """
 
@@ -165,14 +168,16 @@ class StructEpsParams(PatchParamSwitcher):
     eps: float
     eps_max: float
     eps_min: float
+    reverse: bool = True
 
     def get_plot_params(self) -> PatchParams:
         """Returns :class:`PatchParams` based on user-supplied args."""
         eps_min = min(self.eps_min, 1)
         delta_eps = self.eps - eps_min
         delta_eps_max = self.eps_max - eps_min + 1e-5
-        eps_fraction = delta_eps / delta_eps_max
-        color = 1 - eps_fraction
+        color = delta_eps / delta_eps_max
+        if self.reverse:
+            color = 1 - color
         if self.eps == pec_val:
             return PatchParams(facecolor="gold", edgecolor="k", lw=1)
         return PatchParams(facecolor=str(color), edgecolor=str(color), lw=0)
@@ -193,7 +198,6 @@ class SymParams(PatchParamSwitcher):
 
     def get_plot_params(self) -> PatchParams:
         """Returns :class:`PatchParams` based on user-supplied args."""
-        sym_color = "grey"
 
         if self.sym_value == 1:
             sym_color = "lightsteelblue"
@@ -201,7 +205,7 @@ class SymParams(PatchParamSwitcher):
         if self.sym_value == -1:
             sym_color = "rosybrown"
             return PatchParams(alpha=0.6, facecolor=sym_color, edgecolor=sym_color, hatch="--")
-        return PatchParams()
+        raise ValueError(f"sym_value of {self.sym_value} not recognized, must be 1 or -1.")
 
 
 class SimDataGeoParams(PatchParamSwitcher):
@@ -225,8 +229,11 @@ def plotly_sim(sim, x=None, y=None, z=None):
     for struct in sim.structures:
         geo = struct.geometry
         shapes = geo.intersections(x=x, y=y, z=z)
-        mat_index = sim.medium_map[struct.medium]
 
+        if len(shapes) == 0:
+            continue
+
+        mat_index = sim.medium_map[struct.medium]
         params = StructMediumParams(medium=struct.medium, medium_map=sim.medium_map)
         color = params.get_plot_params().facecolor
 
