@@ -47,5 +47,7 @@ RADPERSEC = "rad/sec"
 # large number used for comparing infinity
 LARGE_NUMBER = np.float32(1e20)
 
-# inf = np.inf
-inf = LARGE_NUMBER
+inf = np.inf
+
+# if |np.pi/2 - angle_theta| < GLANCING_CUTOFF in an angled source or in mode spec, raise warning
+GLANCING_CUTOFF = 0.1
