@@ -1,28 +1,34 @@
 """Near field to far field transformation plugin
 """
-from typing import List, Dict
+from typing import List, Dict, Tuple, Union
 import numpy as np
 import xarray as xr
 import pydantic
 
+from rich.progress import track
+
 from ...constants import C_0, ETA_0, HERTZ, MICROMETER
 from ...components.data import SimulationData, FieldData
 from ...components.monitor import FieldMonitor
-from ...components.types import Direction, Axis, Coordinate
+from ...components.types import Direction, Axis, Coordinate, ArrayLike
 from ...components.medium import Medium
+from ...components.base import Tidy3dBaseModel
 from ...log import SetupError, ValidationError
 
 # Default number of points per wavelength in the background medium to use for resampling fields.
 PTS_PER_WVL = 10
 
+# Numpy float array and related array types
+ArrayLikeN2F = Union[float, List[float], ArrayLike]
 
-class Near2FarSurface(pydantic.BaseModel):
+
+class Near2FarSurface(Tidy3dBaseModel):
     """Data structure to store surface monitor data with associated surface current densities."""
 
     monitor: FieldMonitor = pydantic.Field(
         ...,
         title="Field monitor",
-        description="Object of :class:`.FieldMonitor` on which near fields will be sampled and integrated.",
+        description=":class:`.FieldMonitor` on which near fields will be sampled and integrated.",
     )
 
     normal_dir: Direction = pydantic.Field(
@@ -47,20 +53,9 @@ class Near2FarSurface(pydantic.BaseModel):
         return val
 
 
-class Near2Far(pydantic.BaseModel):
+class Near2Far(Tidy3dBaseModel):
     """Near field to far field transformation tool."""
 
-    class Config:
-        """Sets config for all :class:`Near2Far` objects.
-
-        Configuration Options
-        ---------------------
-        arbitrary_types_allowed : bool = True
-            Allow types like numpy arrays.
-        """
-
-        arbitrary_types_allowed = True
-
     sim_data: SimulationData = pydantic.Field(
         ...,
         title="Simulation data",
@@ -83,37 +78,37 @@ class Near2Far(pydantic.BaseModel):
     pts_per_wavelength: int = pydantic.Field(
         PTS_PER_WVL,
         title="Points per wavelength",
-        description="Number of points per wavelength in the background medium with which \
-to discretize the surface monitors for the projection.",
+        description="Number of points per wavelength in the background medium with which "
+        "to discretize the surface monitors for the projection.",
     )
 
     medium: Medium = pydantic.Field(
         None,
         title="Background medium",
-        description="Background medium in which to radiate near fields to far fields. \
-If None, uses the :class:.Simulation background medium.",
+        description="Background medium in which to radiate near fields to far fields. "
+        "If ``None``, uses the :class:.Simulation background medium.",
     )
 
     origin: Coordinate = pydantic.Field(
         None,
         title="Local origin",
-        description="Local origin used for defining observation points. If None, uses the \
-average of the centers of all surface monitors.",
+        description="Local origin used for defining observation points. If ``None``, uses the "
+        "average of the centers of all surface monitors.",
         units=MICROMETER,
     )
 
     currents: Dict[str, xr.Dataset] = pydantic.Field(
         None,
         title="Surface current densities",
-        description="Dictionary mapping monitor name to a dataset storing \
-the surface current densities.",
+        description="Dictionary mapping monitor name to an ``xarray.Dataset`` storing the "
+        "surface current densities.",
     )
 
-    phasor_sign: float = pydantic.Field(
+    phasor_positive_sign: bool = pydantic.Field(
         1,
         title="Phasor convention",
-        description="Fields evolve as exp(jkr) if set to 1, and exp(-jkr) if set to -1. \
-Should not be changed except in special circumstatnces where the exp(-jkr) convention is used.",
+        description="Fields evolve as exp(jkr) if set to 1, and exp(-jkr) if set to -1. "
+        "Should not be changed except in special cases where the exp(-jkr) convention is used.",
     )
 
     @pydantic.validator("origin", always=True)
@@ -133,7 +128,7 @@ Should not be changed except in special circumstatnces where the exp(-jkr) conve
         return val
 
     @property
-    def nk(self) -> [float, float]:
+    def nk(self) -> Tuple[float, float]:
         """Returns the real and imaginary parts of the background medium's refractive index."""
         eps_complex = self.medium.eps_model(self.frequency)
         return self.medium.eps_complex_to_nk(eps_complex)
@@ -183,7 +178,7 @@ Should not be changed except in special circumstatnces where the exp(-jkr) conve
             Background medium in which to radiate near fields to far fields.
             Default: same as the :class:`.Simulation` background medium.
         origin : :class:`.Coordinate`
-            Local origin used for defining observation points. If None, uses the
+            Local origin used for defining observation points. If ``None``, uses the
             average of the centers of all surface monitors.
         """
 
@@ -389,17 +384,17 @@ the number of directions ({len(normal_dirs)})."
 
     # pylint:disable=too-many-locals
     def _radiation_vectors_for_surface(
-        self, theta: float, phi: float, surface: Near2FarSurface, currents: xr.Dataset
+        self, theta: ArrayLikeN2F, phi: ArrayLikeN2F, surface: Near2FarSurface, currents: xr.Dataset
     ):
         """Compute radiation vectors at an angle in spherical coordinates
         for a given set of surface currents and observation angles.
 
         Parameters
         ----------
-        theta : float
-            Polar angle (rad) downward from x=y=0 line relative to the local origin.
-        phi : float
-            Azimuthal (rad) angle from y=z=0 line relative to the local origin.
+        theta : Union[float, List[float], np.ndarray]
+            Polar angles (rad) downward from x=y=0 line relative to the local origin.
+        phi : Union[float, List[float], np.ndarray]
+            Azimuthal (rad) angles from y=z=0 line relative to the local origin.
         surface: :class:`Near2FarSurface`
             :class:`Near2FarSurface` object to use as source of near field.
         currents : xarray.Dataset
@@ -407,77 +402,110 @@ the number of directions ({len(normal_dirs)})."
 
         Returns
         -------
-        tuple[float, float, float, float]
+        tuple(numpy.ndarray[float], numpy.ndarray[float], numpy.ndarray[float], numpy.ndarray[float])
             ``N_theta``, ``N_phi``, ``L_theta``, ``L_phi`` radiation vectors for the given surface.
         """
 
-        # precompute trig functions
-        sin_theta = np.sin(theta)
-        cos_theta = np.cos(theta)
-        sin_phi = np.sin(phi)
-        cos_phi = np.cos(phi)
-
         # make sure that observation points are interpreted w.r.t. the local origin
-        pts = [currents[name] - origin for name, origin in zip(["x", "y", "z"], self.origin)]
+        pts = [currents[name].values - origin for name, origin in zip(["x", "y", "z"], self.origin)]
 
-        k = self.k
-
-        phase_x = np.exp(-self.phasor_sign * 1j * k * pts[0] * sin_theta * cos_phi)
-        phase_y = np.exp(-self.phasor_sign * 1j * k * pts[1] * sin_theta * sin_phi)
-        phase_z = np.exp(-self.phasor_sign * 1j * k * pts[2] * cos_theta)
-        phase = phase_x * phase_y * phase_z
-
-        _, idx_uv = surface.monitor.pop_axis((0, 1, 2), axis=surface.axis)
+        idx_w, idx_uv = surface.monitor.pop_axis((0, 1, 2), axis=surface.axis)
         _, source_names = surface.monitor.pop_axis(("x", "y", "z"), axis=surface.axis)
 
         idx_u, idx_v = idx_uv
         cmp_1, cmp_2 = source_names
 
-        J = [0, 0, 0]
-        M = [0, 0, 0]
+        theta = np.atleast_1d(theta)
+        phi = np.atleast_1d(phi)
+
+        sin_theta = np.sin(theta)
+        cos_theta = np.cos(theta)
+        sin_phi = np.sin(phi)
+        cos_phi = np.cos(phi)
+
+        J = np.zeros((3, len(theta), len(phi)), dtype=complex)
+        M = np.zeros_like(J)
 
         def integrate_2D(function, pts_u, pts_v):
             """Trapezoidal integration in two dimensions."""
             return np.trapz(np.trapz(function, pts_u, axis=0), pts_v, axis=0)
 
-        J[idx_u] = integrate_2D(currents["J" + cmp_1] * phase, pts[idx_u], pts[idx_v])
-        J[idx_v] = integrate_2D(currents["J" + cmp_2] * phase, pts[idx_u], pts[idx_v])
+        phase = [None] * 3
+        propagation_factor = -self.phasor_positive_sign * 1j * self.k
 
-        M[idx_u] = integrate_2D(currents["M" + cmp_1] * phase, pts[idx_u], pts[idx_v])
-        M[idx_v] = integrate_2D(currents["M" + cmp_2] * phase, pts[idx_u], pts[idx_v])
+        def integrate_for_one_theta(i: int):
+            """Perform integration for a given theta angle index"""
+
+            for j in np.arange(len(phi)):
+
+                phase[0] = np.exp(propagation_factor * pts[0] * sin_theta[i] * cos_phi[j])
+                phase[1] = np.exp(propagation_factor * pts[1] * sin_theta[i] * sin_phi[j])
+                phase[2] = np.exp(propagation_factor * pts[2] * cos_theta[i])
+
+                phase_ij = phase[idx_u][:, None] * phase[idx_v][None, :] * phase[idx_w]
+
+                J[idx_u, i, j] = integrate_2D(
+                    currents["J" + cmp_1].values * phase_ij, pts[idx_u], pts[idx_v]
+                )
+                J[idx_v, i, j] = integrate_2D(
+                    currents["J" + cmp_2].values * phase_ij, pts[idx_u], pts[idx_v]
+                )
+
+                M[idx_u, i, j] = integrate_2D(
+                    currents["M" + cmp_1].values * phase_ij, pts[idx_u], pts[idx_v]
+                )
+                M[idx_v, i, j] = integrate_2D(
+                    currents["M" + cmp_2].values * phase_ij, pts[idx_u], pts[idx_v]
+                )
+
+        if len(theta) < 2:
+            integrate_for_one_theta(0)
+        else:
+            for i in track(
+                np.arange(len(theta)),
+                description=f"Processing surface monitor '{surface.monitor.name}'...",
+            ):
+                integrate_for_one_theta(i)
+
+        cos_th_cos_phi = cos_theta[:, None] * cos_phi[None, :]
+        cos_th_sin_phi = cos_theta[:, None] * sin_phi[None, :]
 
         # N_theta (8.33a)
-        N_theta = J[0] * cos_theta * cos_phi + J[1] * cos_theta * sin_phi - J[2] * sin_theta
+        N_theta = J[0] * cos_th_cos_phi + J[1] * cos_th_sin_phi - J[2] * sin_theta[:, None]
 
         # N_phi (8.33b)
-        N_phi = -J[0] * sin_phi + J[1] * cos_phi
+        N_phi = -J[0] * sin_phi[None, :] + J[1] * cos_phi[None, :]
 
         # L_theta  (8.34a)
-        L_theta = M[0] * cos_theta * cos_phi + M[1] * cos_theta * sin_phi - M[2] * sin_theta
+        L_theta = M[0] * cos_th_cos_phi + M[1] * cos_th_sin_phi - M[2] * sin_theta[:, None]
 
         # L_phi  (8.34b)
-        L_phi = -M[0] * sin_phi + M[1] * cos_phi
+        L_phi = -M[0] * sin_phi[None, :] + M[1] * cos_phi[None, :]
 
         return N_theta, N_phi, L_theta, L_phi
 
-    def _radiation_vectors(self, theta: float, phi: float):
+    def _radiation_vectors(self, theta: ArrayLikeN2F, phi: ArrayLikeN2F):
         """Compute radiation vectors at an angle in spherical coordinates.
 
         Parameters
         ----------
-        theta : float
-            Polar angle (rad) downward from x=y=0 line relative to the local origin.
-        phi : float
-            Azimuthal (rad) angle from y=z=0 line relative to the local origin.
+        theta : Union[float, List[float], np.ndarray]
+            Polar angles (rad) downward from x=y=0 line relative to the local origin.
+        phi : Union[float, List[float], np.ndarray]
+            Azimuthal (rad) angles from y=z=0 line relative to the local origin.
 
         Returns
         -------
-        tuple[float, float, float, float]
+        tuple(numpy.ndarray[float], numpy.ndarray[float], numpy.ndarray[float], numpy.ndarray[float])
             ``N_theta``, ``N_phi``, ``L_theta``, ``L_phi`` radiation vectors.
         """
 
         # compute radiation vectors for the dataset associated with each monitor
-        N_theta, N_phi, L_theta, L_phi = 0.0, 0.0, 0.0, 0.0
+        N_theta = np.zeros((len(theta), len(phi)), dtype=complex)
+        N_phi = np.zeros_like(N_theta)
+        L_theta = np.zeros_like(N_theta)
+        L_phi = np.zeros_like(N_theta)
+
         for surface in self.surfaces:
             _N_th, _N_ph, _L_th, _L_ph = self._radiation_vectors_for_surface(
                 theta, phi, surface, self.currents[surface.monitor.name]
@@ -489,24 +517,28 @@ the number of directions ({len(normal_dirs)})."
 
         return N_theta, N_phi, L_theta, L_phi
 
-    def fields_spherical(self, r, theta, phi):
+    def fields_spherical(self, r: float, theta: ArrayLikeN2F, phi: ArrayLikeN2F) -> xr.Dataset:
         """Get fields at a point relative to monitor center in spherical coordinates.
 
         Parameters
         ----------
         r : float
             (micron) radial distance relative to monitor center.
-        theta : float
-            (radian) polar angle downward from x=y=0 relative to the local origin.
-        phi : float
-            (radian) azimuthal angle from y=z=0 line relative to the local origin.
+        theta : Union[float, List[float], np.ndarray]
+            (radian) polar angles downward from x=y=0 relative to the local origin.
+        phi : Union[float, List[float], np.ndarray]
+            (radian) azimuthal angles from y=z=0 line relative to the local origin.
 
         Returns
         -------
-        tuple
-            (Er, Etheta, Ephi), (Hr, Htheta, Hphi), fields in polar coordinates.
+        xarray.Dataset
+            xarray dataset containing (Er, Etheta, Ephi), (Hr, Htheta, Hphi)
+            in polar coordinates.
         """
 
+        theta = np.atleast_1d(theta)
+        phi = np.atleast_1d(phi)
+
         # project radiation vectors to distance r away for given angles
         N_theta, N_phi, L_theta, L_phi = self._radiation_vectors(theta, phi)
 
@@ -514,109 +546,197 @@ the number of directions ({len(normal_dirs)})."
         eta = self.eta
 
         scalar_proj_r = (
-            -self.phasor_sign * 1j * k * np.exp(self.phasor_sign * 1j * k * r) / (4 * np.pi * r)
+            -self.phasor_positive_sign
+            * 1j
+            * k
+            * np.exp(self.phasor_positive_sign * 1j * k * r)
+            / (4 * np.pi * r)
         )
 
         # assemble E felds
-        E_theta = -scalar_proj_r * (L_phi + eta * N_theta)
-        E_phi = scalar_proj_r * (L_theta - eta * N_phi)
-        E_r = np.zeros_like(E_phi)
-        E = np.stack((E_r, E_theta, E_phi))
+        Et_array = -scalar_proj_r * (L_phi + eta * N_theta)
+        Ep_array = scalar_proj_r * (L_theta - eta * N_phi)
+        Er_array = np.zeros_like(Ep_array)
 
         # assemble H fields
-        H_theta = -E_phi / eta
-        H_phi = E_theta / eta
-        H_r = np.zeros_like(H_phi)
-        H = np.stack((H_r, H_theta, H_phi))
+        Ht_array = -Ep_array / eta
+        Hp_array = Et_array / eta
+        Hr_array = np.zeros_like(Hp_array)
 
-        return E, H
+        dims = ("r", "theta", "phi")
+        coords = {"r": [r], "theta": theta, "phi": phi}
 
-    def fields_cartesian(self, x, y, z):
+        Er = xr.DataArray(data=Er_array[None, ...], coords=coords, dims=dims)
+        Et = xr.DataArray(data=Et_array[None, ...], coords=coords, dims=dims)
+        Ep = xr.DataArray(data=Ep_array[None, ...], coords=coords, dims=dims)
+
+        Hr = xr.DataArray(data=Hr_array[None, ...], coords=coords, dims=dims)
+        Ht = xr.DataArray(data=Ht_array[None, ...], coords=coords, dims=dims)
+        Hp = xr.DataArray(data=Hp_array[None, ...], coords=coords, dims=dims)
+
+        field_data = xr.Dataset(
+            {"E_r": Er, "E_theta": Et, "E_phi": Ep, "H_r": Hr, "H_theta": Ht, "H_phi": Hp}
+        )
+
+        return field_data
+
+    def fields_cartesian(self, x: ArrayLikeN2F, y: ArrayLikeN2F, z: ArrayLikeN2F) -> xr.Dataset:
         """Get fields at a point relative to monitor center in cartesian coordinates.
 
         Parameters
         ----------
-        x : float
-            (micron) x position relative to the local origin.
-        y : float
-            (micron) y position relative to the local origin.
-        z : float
-            (micron) z position relative to the local origin.
+        x : Union[float, List[float], np.ndarray]
+            (micron) x positions relative to the local origin.
+        y : Union[float, List[float], np.ndarray]
+            (micron) y positions relative to the local origin.
+        z : Union[float, List[float], np.ndarray]
+            (micron) z positions relative to the local origin.
 
         Returns
         -------
-        tuple
-            (Ex, Ey, Ez), (Hx, Hy, Hz), fields in cartesian coordinates.
+        xarray.Dataset
+            xarray dataset containing (Ex, Ey, Ez), (Hx, Hy, Hz) in cartesian coordinates.
         """
-        r, theta, phi = self._car_2_sph(x, y, z)
-        E, H = self.fields_spherical(r, theta, phi)
-        Er, Etheta, Ephi = E
-        Hr, Htheta, Hphi = H
-        E = Ex, Ey, Ez = self._sph_2_car_field(Er, Etheta, Ephi, theta, phi)
-        H = Hx, Hy, Hz = self._sph_2_car_field(Hr, Htheta, Hphi, theta, phi)
-        return E, H
 
-    def power_spherical(self, r, theta, phi):
+        x, y, z = [np.atleast_1d(x), np.atleast_1d(y), np.atleast_1d(z)]
+
+        Ex_data = np.zeros((len(x), len(y), len(z)), dtype=complex)
+        Ey_data = np.zeros_like(Ex_data)
+        Ez_data = np.zeros_like(Ex_data)
+
+        Hx_data = np.zeros_like(Ex_data)
+        Hy_data = np.zeros_like(Ex_data)
+        Hz_data = np.zeros_like(Ex_data)
+
+        for i in track(np.arange(len(x)), description="Computing far fields"):
+            _x = x[i]
+            for j in np.arange(len(y)):
+                _y = y[j]
+                for k in np.arange(len(z)):
+                    _z = z[k]
+
+                    r, theta, phi = self._car_2_sph(_x, _y, _z)
+                    _field_data = self.fields_spherical(r, theta, phi)
+
+                    Er, Etheta, Ephi = [
+                        _field_data[comp].values for comp in ["E_r", "E_theta", "E_phi"]
+                    ]
+                    Hr, Htheta, Hphi = [
+                        _field_data[comp].values for comp in ["H_r", "H_theta", "H_phi"]
+                    ]
+
+                    Ex_data[i, j, k], Ey_data[i, j, k], Ez_data[i, j, k] = self._sph_2_car_field(
+                        Er, Etheta, Ephi, theta, phi
+                    )
+                    Hx_data[i, j, k], Hy_data[i, j, k], Hz_data[i, j, k] = self._sph_2_car_field(
+                        Hr, Htheta, Hphi, theta, phi
+                    )
+
+        dims = ("x", "y", "z")
+        coords = {"x": x, "y": y, "z": z}
+
+        Ex = xr.DataArray(data=Ex_data, coords=coords, dims=dims)
+        Ey = xr.DataArray(data=Ey_data, coords=coords, dims=dims)
+        Ez = xr.DataArray(data=Ez_data, coords=coords, dims=dims)
+
+        Hx = xr.DataArray(data=Hx_data, coords=coords, dims=dims)
+        Hy = xr.DataArray(data=Hy_data, coords=coords, dims=dims)
+        Hz = xr.DataArray(data=Hz_data, coords=coords, dims=dims)
+
+        field_data = xr.Dataset({"Ex": Ex, "Ey": Ey, "Ez": Ez, "Hx": Hx, "Hy": Hy, "Hz": Hz})
+
+        return field_data
+
+    def power_spherical(self, r: float, theta: ArrayLikeN2F, phi: ArrayLikeN2F) -> xr.DataArray:
         """Get power scattered to a point relative to the local origin in spherical coordinates.
 
         Parameters
         ----------
         r : float
             (micron) radial distance relative to the local origin.
-        theta : float
-            (radian) polar angle downward from x=y=0 relative to the local origin.
-        phi : float
-            (radian) azimuthal angle from y=z=0 line relative to the local origin.
+        theta : Union[float, List[float], np.ndarray]
+            (radian) polar angles downward from x=y=0 relative to the local origin.
+        phi : Union[float, List[float], np.ndarray]
+            (radian) azimuthal angles from y=z=0 line relative to the local origin.
 
         Returns
         -------
-        float
-            Power at point relative to the local origin.
+        power : xarray.DataArray
+            Power at points relative to the local origin.
         """
-        E, H = self.fields_spherical(r, theta, phi)
-        _, E_theta, E_phi = E
-        _, H_theta, H_phi = H
+
+        theta = np.atleast_1d(theta)
+        phi = np.atleast_1d(phi)
+
+        field_data = self.fields_spherical(r, theta, phi)
+        E_theta, E_phi = [field_data[comp].values for comp in ["E_theta", "E_phi"]]
+        H_theta, H_phi = [field_data[comp].values for comp in ["H_theta", "H_phi"]]
         power_theta = 0.5 * np.real(E_theta * np.conj(H_phi))
         power_phi = 0.5 * np.real(-E_phi * np.conj(H_theta))
-        return power_theta + power_phi
+        power_data = power_theta + power_phi
 
-    def power_cartesian(self, x, y, z):
+        dims = ("r", "theta", "phi")
+        coords = {"r": [r], "theta": theta, "phi": phi}
+
+        return xr.DataArray(data=power_data, coords=coords, dims=dims)
+
+    def power_cartesian(self, x: ArrayLikeN2F, y: ArrayLikeN2F, z: ArrayLikeN2F) -> xr.DataArray:
         """Get power scattered to a point relative to the local origin in cartesian coordinates.
 
         Parameters
         ----------
-        x : float
-            (micron) x distance relative to the local origin.
-        y : float
-            (micron) y distance relative to the local origin.
-        z : float
-            (micron) z distance relative to the local origin.
+        x : Union[float, List[float], np.ndarray]
+            (micron) x distances relative to the local origin.
+        y : Union[float, List[float], np.ndarray]
+            (micron) y distances relative to the local origin.
+        z : Union[float, List[float], np.ndarray]
+            (micron) z distances relative to the local origin.
 
         Returns
         -------
-        float
-            Power at point relative to the local origin.
+        power : xarray.DataArray
+            Power at points relative to the local origin.
         """
-        r, theta, phi = self._car_2_sph(x, y, z)
-        return self.power_spherical(r, theta, phi)
 
-    def radar_cross_section(self, theta, phi):
+        x, y, z = [np.atleast_1d(x), np.atleast_1d(y), np.atleast_1d(z)]
+
+        power_data = np.zeros((len(x), len(y), len(z)))
+
+        for i in track(np.arange(len(x)), description="Computing far field power"):
+            _x = x[i]
+            for j in np.arange(len(y)):
+                _y = y[j]
+                for k in np.arange(len(z)):
+                    _z = z[k]
+
+                    r, theta, phi = self._car_2_sph(_x, _y, _z)
+                    power_data[i, j, k] = self.power_spherical(r, theta, phi).values
+
+        dims = ("x", "y", "z")
+        coords = {"x": x, "y": y, "z": z}
+
+        return xr.DataArray(data=power_data, coords=coords, dims=dims)
+
+    def radar_cross_section(self, theta: ArrayLikeN2F, phi: ArrayLikeN2F) -> xr.DataArray:
         """Get radar cross section at a point relative to the local origin in
         units of incident power.
 
         Parameters
         ----------
-        theta : float
-            (radian) polar angle downward from x=y=0 relative to the local origin.
-        phi : float
-            (radian) azimuthal angle from y=z=0 line relative to the local origin.
+        theta : Union[float, List[float], np.ndarray]
+            (radian) polar angles downward from x=y=0 relative to the local origin.
+        phi : Union[float, List[float], np.ndarray]
+            (radian) azimuthal angles from y=z=0 line relative to the local origin.
 
         Returns
         -------
-        RCS : float
+        RCS : xarray.DataArray
             Radar cross section at angles relative to the local origin.
         """
 
+        theta = np.atleast_1d(theta)
+        phi = np.atleast_1d(phi)
+
         _, index_k = self.nk
         if index_k != 0.0:
             raise SetupError("Can't compute RCS for a lossy background medium.")
@@ -631,7 +751,12 @@ the number of directions ({len(normal_dirs)})."
         constant = k**2 / (8 * np.pi * eta)
         term1 = np.abs(L_phi + eta * N_theta) ** 2
         term2 = np.abs(L_theta - eta * N_phi) ** 2
-        return constant * (term1 + term2)
+        RCS_data = constant * (term1 + term2)
+
+        dims = ("theta", "phi")
+        coords = {"theta": theta, "phi": phi}
+
+        return xr.DataArray(data=RCS_data, coords=coords, dims=dims)
 
     @staticmethod
     def _car_2_sph(x, y, z):
