@@ -1,9 +1,10 @@
 """ sets configuration options for web interface """
+import os
 from typing import Any
 
 from dataclasses import dataclass
 
-SOLVER_VERSION = "release-22.1.5"
+SOLVER_VERSION = "release-22.1.6"
 
 
 @dataclass
@@ -29,6 +30,13 @@ ConfigDev = WebConfig(
     web_api_endpoint="https://tidy3d-api.dev-simulation.cloud",
 )
 
+# staging config
+ConfigUat = WebConfig(
+    s3_region="us-gov-west-1",
+    studio_bucket="flow360studio",
+    auth_api_endpoint="https://portal-api.simulation.cloud",
+    web_api_endpoint="https://uat-tidy3d-api.simulation.cloud",
+)
 
 # production config
 ConfigProd = WebConfig(
@@ -38,6 +46,12 @@ ConfigProd = WebConfig(
     web_api_endpoint="https://tidy3d-api.simulation.cloud",
 )
 
-
 # default one to import
 DEFAULT_CONFIG = ConfigProd
+
+if os.environ.get("TIDY3D_ENV") == "dev":
+    DEFAULT_CONFIG = ConfigDev
+elif os.environ.get("TIDY3D_ENV") == "uat":
+    DEFAULT_CONFIG = ConfigUat
+elif os.environ.get("TIDY3D_ENV") == "prod":
+    DEFAULT_CONFIG = ConfigProd
