@@ -13,6 +13,7 @@ from rich.progress import Progress
 
 from .config import DEFAULT_CONFIG as Config
 from .s3utils import get_s3_user, DownloadProgress
+from .auth import requires_auth
 from .task import TaskId, TaskInfo
 from . import httputils as http
 from ..components.simulation import Simulation
@@ -51,7 +52,7 @@ def run(  # pylint:disable=too-many-arguments
     normalize_index : int = 0
         If specified, normalizes the frequency-domain data by the amplitude spectrum of the source
         corresponding to ``simulation.sources[normalize_index]``.
-        This occurs when the data is loaded into a :class:`SimulationData` object.
+        This occurs when the data is loaded into a :class:`.SimulationData` object.
         To turn off normalization, set ``normalize_index`` to ``None``.
 
     Returns
@@ -110,6 +111,7 @@ def upload(
     return task_id
 
 
+@requires_auth
 def get_info(task_id: TaskId) -> TaskInfo:
     """Return information about a task.
 
@@ -130,6 +132,7 @@ def get_info(task_id: TaskId) -> TaskInfo:
     return TaskInfo(**info_dict)
 
 
+@requires_auth
 def start(task_id: TaskId) -> None:
     """Start running the simulation associated with task.
 
@@ -149,6 +152,7 @@ def start(task_id: TaskId) -> None:
     http.put(method, data=task.dict())
 
 
+@requires_auth
 def get_run_info(task_id: TaskId):
     """Gets the % done and field_decay for a running task.
 
@@ -160,19 +164,24 @@ def get_run_info(task_id: TaskId):
     Returns
     -------
     perc_done : float
-        Percentage of run done (in terms of max number of time steps)
+        Percentage of run done (in terms of max number of time steps).
+        Is ``None`` if run info not available.
     field_decay : float
         Average field intensity normlized to max value (1.0).
+        Is ``None`` if run info not available.
     """
-
-    client, bucket, user_id = get_s3_user()
-    key = f"users/{user_id}/{task_id}/output/solver_progress.csv"
-    progress = client.get_object(Bucket=bucket, Key=key)["Body"]
-    progress_string = progress.read().split(b"\n")
-    perc_done, field_decay = progress_string[-2].split(b",")
-    return float(perc_done), float(field_decay)
+    try:
+        client, bucket, user_id = get_s3_user()
+        key = f"users/{user_id}/{task_id}/output/solver_progress.csv"
+        progress = client.get_object(Bucket=bucket, Key=key)["Body"]
+        progress_string = progress.read().split(b"\n")
+        perc_done, field_decay = progress_string[-2].split(b",")
+        return float(perc_done), float(field_decay)
+    except Exception:  # pylint:disable=broad-except
+        return None, None
 
 
+@requires_auth
 def monitor(task_id: TaskId) -> None:
     """Print the real time task progress until completion.
 
@@ -188,58 +197,65 @@ def monitor(task_id: TaskId) -> None:
 
     task_info = get_info(task_id)
     task_name = task_info.taskName
-    status = task_info.status
+
+    status = None
+
+    break_statuses = ("success", "error", "diverged", "deleted", "draft")
+
+    def get_status():
+        """Get status for this task (called many times below, so put into function)."""
+        status = get_info(task_id).status
+        if status == "visualize":
+            return "success"
+        return status
 
     console = Console()
 
-    with console.status(f"[bold green]Working on '{task_name}'...", spinner="runner"):
-        console.log(f"status = {status}")
+    # already done
+    if get_status() in break_statuses:
+        console.log(f"status = {get_status()}")
+        return
 
-        while status not in ("success", "error", "diverged", "deleted", "draft"):
-            new_status = get_info(task_id).status
-            if new_status == "visualize":
-                new_status = "success"
-            if new_status != status:
-                console.log(f"status = {new_status}")
-                status = new_status
+    # preprocessing
+    with console.status(f"[bold green]Starting '{task_name}'...", spinner="runner"):
+        while get_status() not in break_statuses and get_status() != "running":
+            if status != get_status():
+                status = get_status()
+                if status != "running":
+                    console.log(f"status = {status}")
             time.sleep(REFRESH_TIME)
 
-    # # below is the "running" progressbar, needs some work on backend before it's ready.
-    # # TODO: toggle console / display on or off, might want off for Job / Batch to override
-    # perc_done = 0.0
-    # field_decay = 1.0
-    # status = ""
-    # with Progress() as progress:
-    #     def get_description(status: str, num_dots=0) -> str:
-    #         """ gets the progressbar description as a function of status """
-    #         dot_string = ''.join([' ' if i >= num_dots else '.' for i in range(TOTAL_DOTS)])
-    #         base = f"[purple]Monitoring task{dot_string}  "
-    #         if status:
-    #             return base + f"status='{status}'"
-    #         return base
-    #     pbar = progress.add_task(f"[purple]Working on task: '{task_name}'", total=100.0)
-    #     num_dots = 0
-    #     while status not in ("success", "error", "diverged", "deleted", "draft"):
-    #         new_status = get_info(task_id).status
-    #         if new_status != status:
-    #             progress.update(pbar, description=get_description(new_status, num_dots))
-    #             status = new_status
-    #         time.sleep(REFRESH_TIME)
-    #         num_dots = (num_dots + 1) % (TOTAL_DOTS + 1)
-    #         progress.update(pbar, description=get_description(status, num_dots))
-    #         if new_status in ("running", ):
-    #             # try getting new percentage, if not available, just make some up
-    #             try:
-    #                 perc_done_new, field_decay_new = get_run_info(task_id)
-    #             # TODO: get the perc right away so we dont have to handle this.
-    #             except Exception as e:
-    #                 perc_done_new = perc_done
-    #                 field_decay_new = field_decay
-    #             # advance the progressbar
-    #             progress.update(pbar, description=get_description(new_status, num_dots),
-    #                  advance=perc_done_new - perc_done)
-    #             perc_done = perc_done_new
-    #             field_decay = field_decay_new
+    # startup phase where run info is not available
+    console.log("starting up solver")
+    while get_run_info(task_id)[0] is None and get_status() == "running":
+        time.sleep(REFRESH_TIME)
+
+    # phase where run % info is available
+    console.log("running solver")
+    with Progress(console=console) as progress:
+        pbar_pd = progress.add_task("% done", total=100)
+        perc_done, _ = get_run_info(task_id)
+        while perc_done is not None and perc_done < 100 and get_status() == "running":
+            perc_done, field_decay = get_run_info(task_id)
+            new_description = f"% done (field decay = {field_decay:.2e})"
+            progress.update(pbar_pd, completed=perc_done, description=new_description)
+            time.sleep(1.0)
+        if perc_done is not None and perc_done < 100:
+            console.log("early shutoff detected, exiting.")
+        else:
+            progress.update(pbar_pd, completed=100)
+
+    # postprocessing
+    with console.status(f"[bold green]Finishing '{task_name}'...", spinner="runner"):
+        while get_status() not in break_statuses:
+            if status != get_status():
+                status = get_status()
+                console.log(f"status = {status}")
+            time.sleep(REFRESH_TIME)
+
+    # final status (if diffrent from the last printed status)
+    if status != get_status():
+        console.log(f"status = {get_status()}")
 
 
 def download(task_id: TaskId, path: str = "simulation_data.hdf5") -> None:
@@ -288,7 +304,7 @@ def load(
     normalize_index : int = 0
         If specified, normalizes the frequency-domain data by the amplitude spectrum of the source
         corresponding to ``simulation.sources[normalize_index]``.
-        This occurs when the data is loaded into a :class:`SimulationData` object.
+        This occurs when the data is loaded into a :class:`.SimulationData` object.
         To turn off normalization, set ``normalize_index`` to ``None``.
 
     Returns
@@ -318,6 +334,7 @@ def load(
     return sim_data
 
 
+@requires_auth
 def delete(task_id: TaskId) -> TaskInfo:
     """Delete server-side data associated with task.
 
@@ -336,6 +353,7 @@ def delete(task_id: TaskId) -> TaskInfo:
     return http.delete(method)
 
 
+@requires_auth
 def delete_old(days_old: int = 100, folder: str = None) -> int:
     """Delete all tasks older than a given amount of days.
 
@@ -369,6 +387,7 @@ def delete_old(days_old: int = 100, folder: str = None) -> int:
     return count
 
 
+@requires_auth
 def get_tasks(num_tasks: int = None, order: Literal["new", "old"] = "new") -> List[Dict]:
     """Get a list with the metadata of the last ``num_tasks`` tasks.
 
@@ -413,6 +432,7 @@ def get_tasks(num_tasks: int = None, order: Literal["new", "old"] = "new") -> Li
     return out_dict
 
 
+@requires_auth
 def _upload_task(  # pylint:disable=too-many-locals,too-many-arguments
     simulation: Simulation,
     task_name: str,
@@ -423,7 +443,7 @@ def _upload_task(  # pylint:disable=too-many-locals,too-many-arguments
 ) -> TaskId:
     """upload with all kwargs exposed"""
 
-    simulation.validate_contents()
+    simulation.validate_pre_upload()
 
     json_string = simulation._json_string()  # pylint:disable=protected-access
     data = {
@@ -469,6 +489,7 @@ def _upload_task(  # pylint:disable=too-many-locals,too-many-arguments
     return task_id
 
 
+@requires_auth
 def _download_file(task_id: TaskId, fname: str, path: str) -> None:
     """Download a specific file from server.
 
